<?php
namespace Financeiro\Repositories;

class Contareceber extends \Doctrine\ORM\EntityRepository
{

    public function consultaQuantidadeDuplicata($cdfilial, $nrlanctonf)
    {
        $sql = <<<SQL
        SELECT COUNT(*) AS QTDDUPLICATA
          FROM CONTARECEBER
         WHERE CDFILIAL = :CDFILIAL
           AND NRLANCTONF = :NRLANCTONF
SQL;
        $params = array(
            "CDFILIAL"   => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaContarecerNFE($cdfilial, $nrlanctonf)
    {
        $sql = <<<SQL
        SELECT NRSEQVENREC, DTORIGVENREC,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRORIGREC, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRORIGREC, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRORIGREC, 2), '9999999999999.99'))) AS VRORIGREC
          FROM CONTARECEBER
         WHERE CDFILIAL = :CDFILIAL
           AND NRLANCTONF = :NRLANCTONF
        ORDER BY NRSEQVENREC
SQL;

        $params = array(
            "CDFILIAL"   => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrorg
     * @return \Doctrine\DBAL\Driver\Statement
     */
    public function deletaDadosNotaFiscal($cdfilial, $nrlanctonf, $nrorg)
    {
        $query = 'DELETE FROM '.self::getClassMetadata()->getTableName().' WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF AND NRORG = :NRORG';
        $params = array(
            'CDFILIAL' => $cdfilial,
            'NRLANCTONF' => $nrlanctonf,
            'NRORG' => $nrorg
        );

        return self::getEntityManager()->getConnection()->executeQuery($query, $params);
    }
    
    public function deletaContasReceber($cdfilial, $nrlanctonf, $nrorg, $nrlanbolrec)
    {
    	$query = 'DELETE FROM '.self::getClassMetadata()->getTableName().' WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF AND NRORG = :NRORG AND NRLANBOLREC = :NRLANBOLREC';
    	$params = array(
    			'CDFILIAL' => $cdfilial,
    			'NRLANCTONF' => $nrlanctonf,
    			'NRORG' => $nrorg,
    			'NRLANBOLREC' => $nrlanbolrec
    	);
    
    	return self::getEntityManager()->getConnection()->executeQuery($query, $params);
    }
    
    public static function consultaNumeroDuplicata($nrlanbolrec, $cdfilial, $nrorg){
        
        $sql = <<<SQL
                    SELECT DECODE(CR.NRPARCELAREC, '00', CR.NRSEQVENREC, CR.NRPARCELAREC) AS NRPARCELA,
                           NVL(DP.NRDUPLICATA, NF.NRNOTAFISC) DUPLICATA,
                           NF.DTEMISSAO
                    FROM DUPLICATA DP, CONTARECEBER CR, NOTAFISCAL NF
                    WHERE (NF.CDFILIAL     = CR.CDFILIAL      ) AND
                          (NF.NRLANCTONF   = CR.NRLANCTONF    ) AND
                          (CR.CDFILIALDUP  = DP.CDFILIAL   (+)) AND
                          (CR.NRLANCTODUP  = DP.NRLANCTODUP(+)) AND
                          (CR.NRLANBOLREC  = :P_NRLANBOLREC   ) AND
                          (CR.CDFILIBOLREC = :P_CDFILIAL      ) AND
                          (CR.NRORG        = :P_NRORG         )
SQL;
        $params = array(
            'P_NRLANBOLREC' => $nrlanbolrec,
            'P_CDFILIAL'    => $cdfilial,
            'P_NRORG'       => $nrorg,
        );
        
        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e){
            throw $e;
        }
        
    }
    
    public static function updateContaReceber($dtatuavenrec, $dscontarec, $vratuarec, $cdfilibolrec, $cdmotiparc, $idstatrec, $nrlanbolrec,
        $cdclassfina, $cdempresa, $cdfimovflrec, $dtmovfluxo, $idmovfluxo, $cdtpctreceb, $dtorigvenrec, $vrorigrec,
        $p_nrparcelarec, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg){
    	$sql = <<<SQL
    				update CONTARECEBER set DTATUAVENREC = :DTATUAVENREC, DSCONTAREC = :DSCONTAREC, 
    					VRATUAREC = :VRATUAREC, CDFILIBOLREC = :CDFILIBOLREC, CDMOTIPARC = :CDMOTIPARC, 
    					IDSTATREC = :IDSTATREC, NRLANBOLREC = :NRLANBOLREC, CDCLASSFINA = :CDCLASSFINA, 
    					CDEMPRESA = :CDEMPRESA, CDFIMOVFLREC = :CDFIMOVFLREC, DTMOVFLUXO = :DTMOVFLUXO, 
    					IDMOVFLUXO = :IDMOVFLUXO, CDTPCTRECEB = :CDTPCTRECEB, DTORIGVENREC = :DTORIGVENREC, 
    					VRORIGREC = :VRORIGREC, NRPARCELAREC = :NRPARCELAREC 
    					where CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF AND NRSEQVENREC = :NRSEQVENREC 
    					AND NRPARCELAREC = :P_NRPARCELAREC AND NRORG = :NRORG
SQL;
    	$params = array(
    		'DTATUAVENREC' => $dtatuavenrec,
    		'DSCONTAREC' => $dscontarec,
    		'VRATUAREC' => $vratuarec,
    		'CDFILIBOLREC' => $cdfilibolrec,
    		'CDMOTIPARC' => $cdmotiparc,
    		'IDSTATREC' => $idstatrec,
    		'NRLANBOLREC' => $nrlanbolrec,
    		'CDCLASSFINA' => $cdclassfina,
    		'CDEMPRESA' => $cdempresa,
    		'CDFIMOVFLREC' => $cdfimovflrec,
    		'DTMOVFLUXO' => $dtmovfluxo,
    		'IDMOVFLUXO' => $idmovfluxo,
    		'CDTPCTRECEB' => $cdtpctreceb,
    		'DTORIGVENREC' => $dtorigvenrec,
    		'VRORIGREC' => $vrorigrec,
    		'CDFILIAL' => $cdfilial,
    		'NRLANCTONF' => $nrlanctonf,
    		'NRSEQVENREC' => $nrseqvenrec,
    		'P_NRPARCELAREC' => $p_nrparcelarec,
    		'NRPARCELAREC' => $nrparcelarec,
    		'NRORG' => $nrorg,
    	);
    	
    	try{
    		return self::getEntityManager()->getConnection()->executeQuery($sql, $params);
    	} catch(\Exception $e){
    		throw $e;
    	}
    }

    public static function updateContaBaixadaPorRetorno($cdfilial, $nrbaixarec){
        $sql = <<<SQL
                  UPDATE CONTARECEBER
                    SET
                       IDSTATREC = 'P'
                    WHERE
                       (CDFILIBOLREC||NRLANBOLREC IN (SELECT CDFILIAL||NRLANBOLREC
                                                      FROM BOLETRECEBER
                                                      WHERE
                                                         CDFILIBAIREC = :P_CDFILIBAIREC AND
                                                         NRBAIXAREC   = :P_NRBAIXAREC))
SQL;
        $params = array(
            'P_CDFILIBAIREC' => $cdfilial,
            'P_NRBAIXAREC'   => $nrbaixarec
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e){
            throw $e;
        }

    }

    public function getMaxNRSEQVENREC($filial, $lancamento){
        $sql = <<<SQL
        SELECT NVL(MAX(NRSEQVENREC),'00') AS NRSEQVENREC
          FROM CONTARECEBER
         WHERE CDFILIAL   = :P_CDFILIAL
           AND NRLANCTONF = :P_NRLANCTONF
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
	
	public function getPriUltCompra($cdcliente, $dtinipos, $dtfinpos, $dtsaida){
		$sql = <<<SQL
			SELECT DISTINCT
			   C.CDCLIENTE,
			   C.NMRAZSOCCLIE,
			   NVL(PC.PRIMCOMPRA,0) AS PRIMCOMPRA,
			   PC.PRIMDATA,
			   NVL(UC.ULTCOMPRA,0) AS ULTCOMPRA,
			   UC.DTEMISSAO AS ULTDATA,
			   NVL(TC.TOTALCOMPRA,0) AS TOTALCOMPRA
			FROM
			   (SELECT
				   NF.CDCLIENTE,
				   SUM(CR.VRORIGREC) AS PRIMCOMPRA,
				   NF.DTEMISSAO        AS PRIMDATA,
				   NF.IDENTRSAIDOP
				FROM
				   NOTAFISCAL NF, CONTARECEBER CR
				WHERE
				   (NF.IDSTATUS     IN ( 'A', 'E'))  AND
				   (NF.IDENTRSAIDOP = 'S') AND
				   (NF.CDFILIAL = CR.CDFILIAL) AND
				   (NF.NRLANCTONF = CR.NRLANCTONF) AND
				   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
				   (NF.CDCLIENTE = :P_CDCLIENTE)     AND
				   (NF.NRLANCTONF = (SELECT
									MIN(N.NRLANCTONF)
								 FROM
									NOTAFISCAL N, CONTARECEBER C
								 WHERE
									(N.IDENTRSAIDOP = 'S') AND
									(N.IDSTATUS IN ( 'A', 'E')) AND
									(N.CDFILIAL = C.CDFILIAL) AND
									(N.NRLANCTONF = C.NRLANCTONF) AND
									((N.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
									(N.CDCLIENTE =  NF.CDCLIENTE)))
				GROUP BY
				   NF.CDCLIENTE,
				   NF.DTEMISSAO,
				   NF.IDENTRSAIDOP) PC,

			   (SELECT
				   NF.CDCLIENTE,
				   SUM(CR.VRORIGREC)      AS ULTCOMPRA,
				   NF.DTEMISSAO,
				   NF.IDENTRSAIDOP
				FROM
				   NOTAFISCAL NF, CONTARECEBER CR
				WHERE
				   (NF.IDENTRSAIDOP = 'S' )           AND
				   (NF.IDSTATUS     IN ( 'A', 'E'))   AND
				   (NF.CDFILIAL = CR.CDFILIAL) AND
				   (NF.NRLANCTONF = CR.NRLANCTONF) AND
				   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
				   (NF.CDCLIENTE = :P_CDCLIENTE) AND
				   (NF.NRLANCTONF = (SELECT
									MAX(N.NRLANCTONF)
								 FROM
									NOTAFISCAL N, CONTARECEBER C
								 WHERE
									(N.IDENTRSAIDOP = 'S') AND
									(N.IDSTATUS IN ( 'A', 'E')) AND
									(N.CDFILIAL = C.CDFILIAL) AND
									(N.NRLANCTONF = C.NRLANCTONF) AND
									((N.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
									(N.CDCLIENTE = NF.CDCLIENTE)))
				GROUP BY
				   NF.CDCLIENTE,
				   NF.DTEMISSAO,
				   NF.IDENTRSAIDOP) UC,
				(SELECT
				   NF.CDCLIENTE,
				   NF.IDENTRSAIDOP,
				   SUM(CR.VRORIGREC) AS TOTALCOMPRA
				FROM
				   NOTAFISCAL NF, CONTARECEBER CR
				WHERE
				   (NF.IDSTATUS     IN ( 'A', 'E'))      AND
				   (NF.IDENTRSAIDOP = 'S') AND
				   (NF.CDFILIAL = CR.CDFILIAL) AND
				   (NF.NRLANCTONF = CR.NRLANCTONF) AND
				   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
				   (NF.CDCLIENTE = :P_CDCLIENTE)
				GROUP BY NF.CDCLIENTE, NF.IDENTRSAIDOP) TC,
			   CLIENTE C,
			   NOTAFISCAL NF
			WHERE
			   (C.CDCLIENTE =  :P_CDCLIENTE) AND
			   (C.CDCLIENTE = NF.CDCLIENTE )          AND
			   (C.CDCLIENTE = PC.CDCLIENTE   (+))     AND
			   (NF.IDENTRSAIDOP = PC.IDENTRSAIDOP)    AND
			   (NF.IDSTATUS     IN ( 'A', 'E'))       AND
			   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
			   (NF.IDENTRSAIDOP = 'S')                AND
			   (C.CDCLIENTE     = UC.CDCLIENTE   (+)) AND
			   (NF.IDENTRSAIDOP = UC.IDENTRSAIDOP)    AND
			   (C.CDCLIENTE = TC.CDCLIENTE   (+))     AND
			   (NF.IDENTRSAIDOP = TC.IDENTRSAIDOP)
			GROUP BY
			   C.CDCLIENTE,
			   C.NMRAZSOCCLIE,
			   PC.PRIMCOMPRA,
			   PC.PRIMDATA,
			   UC.ULTCOMPRA,
			   UC.DTEMISSAO,
			   TC.TOTALCOMPRA
			ORDER BY
			   C.NMRAZSOCCLIE
SQL;

        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_DTSINI"       => $dtinipos,
            "P_DTSFIM"       => $dtfinpos,
            "P_DTTDATASAIDA" => $dtsaida
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
	}

    public function getTotais($cdcliente, $dtinipos, $dtfinpos, $dtsaida){
        $sql = <<<SQL
			SELECT DISTINCT
               NVL(TCP.TOTCOMPAG,0) AS TOTCOMPAG,
               C.CDCLIENTE,
               C.NMRAZSOCCLIE,
               NVL(TCA.TOTATR,0) AS TOTATR,
               NVL(TCV.TOTVENC,0) AS TOTVENC,
               NVL(TCAV.TOTAVENC,0) AS TOTAVENC
            FROM
              (SELECT
                   NF.CDCLIENTE,
                   SUM(CR.VRORIGREC) AS TOTCOMPAG
                FROM
                   CONTARECEBER CR,
                   NOTAFISCAL NF
                WHERE
                   (CR.IDSTATREC    = 'B')          AND
                   (NF.IDSTATUS     IN ('A', 'E'))  AND
                   (NF.IDENTRSAIDOP = 'S')          AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (NF.CDCLIENTE    = :P_CDCLIENTE) AND
                   (NF.CDFILIAL     = CR.CDFILIAL ) AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF)
                GROUP BY NF.CDCLIENTE) TCP,
                (SELECT
                   NF.CDCLIENTE,
                   SUM(CR.VRORIGREC) AS TOTATR,
                   NF.IDENTRSAIDOP
                FROM
                   NOTAFISCAL NF,
                   CONTARECEBER CR
                WHERE
                   (CR.IDSTATREC   <> 'B')             AND
                   (NF.IDENTRSAIDOP = 'S')             AND
                   (NF.IDSTATUS     IN ( 'A', 'E'))    AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (NF.CDFILIAL     = CR.CDFILIAL )    AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF)   AND
                   (NF.CDCLIENTE    = :P_CDCLIENTE)       AND
                   (CR.DTATUAVENREC < TO_CHAR(SYSDATE, 'DD/MM/YYYY'))
                GROUP BY
                   NF.CDCLIENTE,
                   NF.IDENTRSAIDOP)TCA,
             (SELECT
                   NF.CDCLIENTE,
                   SUM(CR.VRORIGREC) AS TOTVENC,
                   NF.IDENTRSAIDOP
                FROM
                   NOTAFISCAL NF,
                   CONTARECEBER CR
                WHERE
                   (CR.IDSTATREC   <> 'B')            AND
                   (NF.IDSTATUS     IN ( 'A', 'E'))   AND
                   (NF.IDENTRSAIDOP = 'S')            AND
                   (NF.CDFILIAL     = CR.CDFILIAL )   AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF)  AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (CR.DTATUAVENREC  = TO_CHAR(SYSDATE, 'DD/MM/YYYY'))   AND
                   (NF.CDCLIENTE = :P_CDCLIENTE)
                GROUP BY
                   NF.CDCLIENTE,
                   NF.IDENTRSAIDOP) TCV,
                (SELECT
                   NF.CDCLIENTE,
                   SUM(CR.VRORIGREC) AS TOTAVENC,
                   NF.IDENTRSAIDOP
                FROM
                   NOTAFISCAL NF,
                   CONTARECEBER CR
                WHERE
                   (CR.IDSTATREC   <> 'B')            AND
                   (NF.IDENTRSAIDOP = 'S')            AND
                   (NF.IDSTATUS     IN ( 'A', 'E'))   AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (NF.CDFILIAL     = CR.CDFILIAL )   AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF)  AND
                   (CR.DTATUAVENREC  >= TO_CHAR(SYSDATE, 'DD/MM/YYYY'))   AND
                   (NF.CDCLIENTE = :P_CDCLIENTE)
                GROUP BY
                   NF.CDCLIENTE,
                   NF.IDENTRSAIDOP) TCAV,
               CLIENTE C,
               NOTAFISCAL NF
            WHERE
               (C.CDCLIENTE = :P_CDCLIENTE) AND
               (C.CDCLIENTE = NF.CDCLIENTE) AND
               (C.CDCLIENTE = TCP.CDCLIENTE  (+)) AND
               (C.CDCLIENTE = TCA.CDCLIENTE  (+)) AND
               (C.CDCLIENTE = TCV.CDCLIENTE  (+)) AND
               (C.CDCLIENTE = TCAV.CDCLIENTE (+)) AND
               (NF.IDSTATUS IN ( 'A', 'E' )     ) AND
               ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
               (NF.IDENTRSAIDOP = 'S')
            GROUP BY
               C.CDCLIENTE,
               C.NMRAZSOCCLIE,
               TCP.TOTCOMPAG,
               TCA.TOTATR,
               TCV.TOTVENC,
               TCAV.TOTAVENC
            ORDER BY
               C.NMRAZSOCCLIE
SQL;

        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_DTSINI"       => $dtinipos,
            "P_DTSFIM"       => $dtfinpos,
            "P_DTTDATASAIDA" => $dtsaida
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getMaiComp($cdcliente, $dtinipos, $dtfinpos, $dtsaida){
        $sql = <<<SQL
			SELECT
               C.CDCLIENTE,
               C.NMRAZSOCCLIE,
               MP.MAIORDATA,
               NVL(MP.MAIORVALOR,0) AS MAIORVALOR
            FROM
                (SELECT
                   CDCLIENTE,
                   MAX(DTEMISSAO)   AS MAIORDATA,
                   VRNOTAFISC       AS MAIORVALOR,
                   IDENTRSAIDOP
                FROM
                   NOTAFISCAL NF, CONTARECEBER CR
                WHERE
                   (IDENTRSAIDOP  = 'S')           AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (NF.CDCLIENTE  = :P_CDCLIENTE)  AND
                   (NF.CDFILIAL   = CR.CDFILIAL)   AND
                   (NF.NRLANCTONF = CR.NRLANCTONF) AND
                   (VRNOTAFISC = (SELECT
                                    MAX(N.VRNOTAFISC)
                                 FROM
                                    NOTAFISCAL N, CONTARECEBER C
                                 WHERE
                                    (N.IDENTRSAIDOP = 'S')        AND
                                    (N.IDSTATUS   IN ( 'A', 'E')) AND
                                    (N.CDFILIAL   = C.CDFILIAL)   AND
                                    (N.NRLANCTONF = C.NRLANCTONF) AND
                                    ((N.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                                    (N.CDCLIENTE  =  NF.CDCLIENTE)))
                GROUP BY
                   CDCLIENTE,
                   VRNOTAFISC,
                   IDENTRSAIDOP) MP,
               CLIENTE C,
               NOTAFISCAL NF
            WHERE
               (C.CDCLIENTE     = :P_CDCLIENTE) AND
               (C.CDCLIENTE     = NF.CDCLIENTE ) AND
               (NF.IDENTRSAIDOP = MP.IDENTRSAIDOP) AND
               ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
               (NF.IDSTATUS     IN ( 'A', 'E'))      AND
               (C.CDCLIENTE     = MP.CDCLIENTE   (+))
            GROUP BY
               C.CDCLIENTE,
               C.NMRAZSOCCLIE,
               MP.MAIORDATA,
               MP.MAIORVALOR
SQL;

        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_DTSINI"       => $dtinipos,
            "P_DTSFIM"       => $dtfinpos,
            "P_DTTDATASAIDA" => $dtsaida
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getMaAtrPg($cdcliente, $dtinipos, $dtfinpos, $dtsaida){
        $sql = <<<SQL
			SELECT *
            FROM(SELECT
                DECODE(PAGO.NRDIASATRASO,0,0,SUM(PAGO.VRMATR)) AS VRMATR,
                PAGO.NRDIASATRASO
              FROM
                (SELECT
                    SUM(NF.VRNOTAFISC) VRMATR,
                    ROUND(NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - BO.DTVENBOLREC) DIAS ,
                    DECODE(
                     (ABS(DECODE((NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - CR.DTATUAVENREC),
                            0, 0,
                            (NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - CR.DTATUAVENREC))) + (NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - CR.DTATUAVENREC)) ,
                      0, 0,
                      (NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - CR.DTATUAVENREC)) AS NRDIASATRASO
                 FROM
                    NOTAFISCAL NF,
                    CONTARECEBER CR,
                    BOLETRECEBER BO,
                    BAIXARECEBIDA BR
                 WHERE
                    (NF.IDENTRSAIDOP = 'S')             AND
                    (CR.IDSTATREC    = 'B')             AND
                    (NF.CDFILIAL     = CR.CDFILIAL )    AND
                    (NF.IDSTATUS     IN ( 'A', 'E'))    AND
                    ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                    (NF.NRLANCTONF   = CR.NRLANCTONF)   AND
                    (CR.CDFILIBOLREC = BO.CDFILIAL)     AND
                    (CR.NRLANBOLREC  = BO.NRLANBOLREC)  AND
                    (BO.CDFILIBAIREC = BR.CDFILIAL)     AND
                    (BO.NRBAIXAREC   = BR.NRBAIXAREC)   AND
                    (NF.CDCLIENTE    =:P_CDCLIENTE)
                 GROUP BY
                     NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - BO.DTVENBOLREC,
                     NVL(BO.DTLIQBOLREC, BR.DTBAIXAREC) - CR.DTATUAVENREC,
                      CR.DTATUAVENREC)PAGO
              GROUP BY
                 PAGO.VRMATR, PAGO.NRDIASATRASO
              ORDER BY
                 PAGO.NRDIASATRASO DESC) A
                 WHERE ROWNUM = 1
SQL;
        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_DTSINI"       => $dtinipos,
            "P_DTSFIM"       => $dtfinpos,
            "P_DTTDATASAIDA" => $dtsaida
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getMaAtrAb($cdcliente, $dtinipos, $dtfinpos, $dtsaida){
        $sql = <<<SQL
			SELECT *
            FROM(
              SELECT
                   TRUNC(MAX((SYSDATE) - (CR.DTATUAVENREC))) AS DATA,
                   NVL(DECODE(TRUNC(MAX((SYSDATE) - (CR.DTATUAVENREC))),
                          0,
                          0,
                          SUM(NF.VRNOTAFISC)
                   ),0) AS VRATRAB,
                   DTVENBOLREC
                FROM
                   NOTAFISCAL NF,
                   CONTARECEBER CR,
                   BOLETRECEBER BO
                WHERE
                   (CR.IDSTATREC   <> 'B')            AND
                   (NF.IDENTRSAIDOP = 'S')            AND
                   (NF.IDSTATUS     IN ( 'A', 'E'))   AND
                   (BO.DTVENBOLREC <= SYSDATE)        AND
                   (NF.CDFILIAL     = CR.CDFILIAL )   AND
                   ((NF.DTENTRSAID BETWEEN :P_DTSINI AND :P_DTSFIM) OR (:P_DTTDATASAIDA = 'T' )) AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF)  AND
                   (CR.CDFILIBOLREC = BO.CDFILIAL)    AND
                   (CR.NRLANBOLREC  = BO.NRLANBOLREC) AND
                   (NF.CDCLIENTE = :P_CDCLIENTE)
                GROUP BY
                   ((SYSDATE) - (CR.DTATUAVENREC)),
                   DTVENBOLREC
                ORDER BY
                   ((SYSDATE) - (CR.DTATUAVENREC)) DESC) A
            WHERE ROWNUM = 1
SQL;
        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_DTSINI"       => $dtinipos,
            "P_DTSFIM"       => $dtfinpos,
            "P_DTTDATASAIDA" => $dtsaida
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getContaAberto($cdcliente, $dtinicial, $dtfinal, $todos){
        $sql = <<<SQL
                SELECT
                   SUM(CR.VRORIGREC) AS VRORIGREC,
                   SUM(CR.VRATUAREC) AS VRATUAREC,
                   NF.NRNOTAFISC,
                   NF.DTEMISSAO,
                   CR.DTATUAVENREC,
                   BR.CDFILIAL,
                   BR.NRLANBOLREC,
                   BR.CDCARTCOBBOL,
                   NVL(VN.NMFANVEN, VE.NMFANVEN) NMFANVEN,
                   DECODE(
                   (ABS(DECODE((TRUNC(SYSDATE) - CR.DTATUAVENREC),
                          0, 0,
                          (TRUNC(SYSDATE) - CR.DTATUAVENREC))) + (TRUNC(SYSDATE) - CR.DTATUAVENREC)) ,
                    0, 0,
                    (TRUNC(SYSDATE) - CR.DTATUAVENREC)) AS NRDIASATRASO
                FROM
                   NOTAFISCAL  NF, CONTARECEBER CR, BOLETRECEBER BR, VENDEDOR VN,
                   (SELECT IT.CDFILIAL, IT.NRLANCTONF, MAX(VE.NMFANVEN) NMFANVEN
                    FROM
                       VENDEDOR VE, ITEMPRODUTO IT, PEDIDO PE
                    WHERE PE.CDCLIENTE   = :P_CDCLIENTE
                      AND IT.CDFILIALPED = PE.CDFILIAL
                      AND IT.NRPEDIDO    = PE.NRPEDIDO
                      AND PE.CDVENDEDOR  = VE.CDVENDEDOR
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF) VE
                WHERE
                   (NF.CDCLIENTE = :P_CDCLIENTE) AND
                   ((CR.DTATUAVENREC BETWEEN :P_DTINICIAL AND :P_DTFINAL) OR (:P_TODOS = 'T')) AND
                   (CR.IDSTATREC    IN ('A', 'D', 'P'))  AND
                   (NF.IDSTATUS     IN ('A', 'E')   )    AND
                   (NF.CDFILIAL     = CR.CDFILIAL   )    AND
                   (NF.NRLANCTONF   = CR.NRLANCTONF )    AND
                   (CR.CDFILIBOLREC = BR.CDFILIAL   )    AND
                   (CR.NRLANBOLREC  = BR.NRLANBOLREC)    AND
                   (NF.CDFILIAL     = VE.CDFILIAL   (+)) AND
                   (NF.NRLANCTONF   = VE.NRLANCTONF (+)) AND
                   (NF.CDVENDEDOR   = VN.CDVENDEDOR (+))
                GROUP BY
                   NF.NRNOTAFISC,
                   NF.DTEMISSAO,
                   CR.DTATUAVENREC,
                   BR.CDFILIAL,
                   BR.NRLANBOLREC,
                   BR.CDCARTCOBBOL,
                   NVL(VN.NMFANVEN, VE.NMFANVEN)
                ORDER BY
                   CR.DTATUAVENREC
SQL;
        $params = array(
            "P_CDCLIENTE" => $cdcliente,
            "P_DTINICIAL" => $dtinicial,
            "P_DTFINAL"   => $dtfinal,
            "P_TODOS"     => $todos
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getTpBolRec($cdcliente, $cdfilial, $dtinicial, $dtfinal, $todos, $nrlanbolrec, $vratuarec){
        $sql = <<<SQL
                SELECT
                   TR.CDTIPORECE,
                   TR.NMTIPORECE,
                   ROUND(:P_VRATUAREC * 100 / SUM(BR.VRFINALBREC) * TB.VRTPRECBOLR / 100) AS VRTPRECBOLR,
                   TB.NRCHEQBOLREC,
                   TB.DTCHEQBOLREC,
                   TB.NRCARCREBOLR,
                   CA.NMCARTAO,
                   BC.CDPICTCONT,
                   (CC.CDBANCO || '/' || CC.CDAGENCIA || '/' || CC.CDCONTCORR) AS CDBANCO,
                   TP.NMTPBAIXAREC
                FROM
                   TPRECBOLETO TB,
                   TIPORECE TR,
                   BOLETRECEBER BR,
                   TIPOBAIXAREC TP,
                   CARTAO CA,
                   CONTCORR CC,
                   BANCO  BC
                WHERE
                   (BR.CDFILIAL      = :P_CDFILIAL   )     AND
                   (BR.NRLANBOLREC   = :P_NRLANBOLREC)     AND
                   (TB.CDFILIAL      = BR.CDFILIAL   )     AND
                   (TB.NRLANBOLREC   = BR.NRLANBOLREC)     AND
                   (TR.CDTIPORECE    = TB.CDTIPORECE )     AND
                   (CA.CDCARTAO (+)  = TB.CDCARTAO   )     AND
                   (BR.CDTPBAIXAREC  = TP.CDTPBAIXAREC(+)) AND
                   (BR.CDREDUCONT    = CC.CDREDUCONT  (+)) AND
                   (CC.CDBANCO       = BC.CDBANCO     (+))
                GROUP BY
                   TR.CDTIPORECE,
                   TR.NMTIPORECE,
                   VRTPRECBOLR,
                   TB.CDTIPORECE,
                   TB.NRCHEQBOLREC,
                   TB.DTCHEQBOLREC,
                   TB.NRCARCREBOLR,
                   CA.NMCARTAO,
                   BC.CDPICTCONT,
                   CC.CDBANCO ,
                   CC.CDAGENCIA ,
                   CC.CDCONTCORR,
                   TP.NMTPBAIXAREC
                ORDER BY
                   TR.CDTIPORECE
SQL;
        $params = array(
            "P_CDCLIENTE"   => $cdcliente,
            "P_CDFILIAL"    => $cdfilial,
            "P_DTINICIAL"   => $dtinicial,
            "P_DTFINAL"     => $dtfinal,
            "P_TODOS"       => $todos,
            "P_NRLANBOLREC" => $nrlanbolrec,
            "P_VRATUAREC"   => $vratuarec
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getContaBaixada($cdcliente, $dtinicial, $dtfinal, $todos){
        $sql = <<<SQL
                SELECT
                   SUM(CR.VRORIGREC) AS VRORIGREC,
                   SUM(CR.VRATUAREC) AS VRATUAREC,
                   NF.NRNOTAFISC,
                   NF.DTEMISSAO,
                   CR.DTATUAVENREC,
                   NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC) DTBAIXAREC,
                   BX.CDFILIAL,
                   BX.NRBAIXAREC,
                   BR.NRLANBOLREC,
                   BR.CDCARTCOBBOL,
                   NVL(VN.NMFANVEN, VE.NMFANVEN) NMFANVEN,
                   DECODE(
                   (ABS(DECODE((NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC) - CR.DTATUAVENREC),
                          0, 0,
                          (NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC) - CR.DTATUAVENREC))) + (NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC) - CR.DTATUAVENREC)) ,
                    0, 0,
                    (NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC) - CR.DTATUAVENREC)) as NRDIASATRASO
                FROM
                   NOTAFISCAL NF, CONTARECEBER CR, BOLETRECEBER BR, BAIXARECEBIDA BX, VENDEDOR VN,
                   (SELECT IT.CDFILIAL, IT.NRLANCTONF, MAX(VE.NMFANVEN) NMFANVEN
                    FROM
                       VENDEDOR VE, ITEMPRODUTO IT, PEDIDO PE
                    WHERE PE.CDCLIENTE   = :P_CDCLIENTE
                      AND IT.CDFILIALPED = PE.CDFILIAL
                      AND IT.NRPEDIDO    = PE.NRPEDIDO
                      AND PE.CDVENDEDOR  = VE.CDVENDEDOR
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF) VE
                WHERE  (NF.CDCLIENTE    = :P_CDCLIENTE  )
                   AND ((CR.DTATUAVENREC BETWEEN :P_DTINICIAL AND :P_DTFINAL) OR (:P_TODOS = 'T'))
                   AND (CR.IDSTATREC    = 'B'           )
                   AND (NF.CDFILIAL     = CR.CDFILIAL   )
                   AND (NF.NRLANCTONF   = CR.NRLANCTONF )
                   AND (CR.CDFILIBOLREC = BR.CDFILIAL   )
                   AND (CR.NRLANBOLREC  = BR.NRLANBOLREC)
                   AND (BR.CDFILIBAIREC = BX.CDFILIAL   )
                   AND (BR.NRBAIXAREC   = BX.NRBAIXAREC )
                   AND (NF.CDFILIAL     = VE.CDFILIAL   (+))
                   AND (NF.NRLANCTONF   = VE.NRLANCTONF (+))
                   AND (NF.CDVENDEDOR   = VN.CDVENDEDOR (+))
                GROUP BY
                   NF.NRNOTAFISC,
                   NF.DTEMISSAO,
                   CR.DTATUAVENREC,
                   NVL(BR.DTLIQBOLREC, BX.DTBAIXAREC),
                   BX.CDFILIAL,
                   BX.NRBAIXAREC,
                   CR.DTATUAVENREC,
                   BR.NRLANBOLREC,
                   BR.CDCARTCOBBOL,
                   NVL(VN.NMFANVEN, VE.NMFANVEN)
                ORDER BY
                   CR.DTATUAVENREC DESC
SQL;
        $params = array(
            "P_CDCLIENTE" => $cdcliente,
            "P_DTINICIAL" => $dtinicial,
            "P_DTFINAL"   => $dtfinal,
            "P_TODOS"     => $todos
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getTpBaixaRec($cdcliente, $cdfilial, $dtinicial, $dtfinal, $todos, $nrlanbolrec, $vratuarec){
        $sql = <<<SQL
                SELECT
                   TR.CDTIPORECE,
                   TR.NMTIPORECE,
                   ROUND(:P_VRATUAREC * 100 / SUM(BR.VRFINALBREC) * TB.VRTPRECBOLR / 100) AS VRTPRECBAIXA,
                   TB.NRCHEQBOLREC   NRCHEQBAIREC,
                   TB.DTCHEQBOLREC   DTCHEQBAIREC,
                   TB.NRCARCREBOLR   NRCARCREBREC,
                   CA.NMCARTAO,
                   BC.CDPICTCONT,
                   (CC.CDBANCO || '/' || CC.CDAGENCIA || '/' || CC.CDCONTCORR) AS CDBANCO,
                   TP.NMTPBAIXAREC
                FROM
                   TPRECBOLETO TB,
                   TIPORECE TR,
                   BOLETRECEBER BR,
                   TIPOBAIXAREC TP,
                   CARTAO CA,
                   CONTCORR CC,
                   BANCO  BC
                WHERE
                   (BR.CDFILIAL      = :P_CDFILIAL   )     AND
                   (BR.NRLANBOLREC   = :P_NRLANBOLREC)     AND
                   (TB.CDFILIAL      = BR.CDFILIAL   )     AND
                   (TB.NRLANBOLREC   = BR.NRLANBOLREC)     AND
                   (TR.CDTIPORECE    = TB.CDTIPORECE )     AND
                   (CA.CDCARTAO (+)  = TB.CDCARTAO   )     AND
                   (BR.CDTPBAIXAREC  = TP.CDTPBAIXAREC(+)) AND
                   (BR.CDREDUCONT    = CC.CDREDUCONT  (+)) AND
                   (CC.CDBANCO       = BC.CDBANCO     (+))
                GROUP BY
                   TR.CDTIPORECE,
                   TR.NMTIPORECE,
                   VRTPRECBOLR,
                   TB.CDTIPORECE,
                   TB.NRCHEQBOLREC,
                   TB.DTCHEQBOLREC,
                   TB.NRCARCREBOLR,
                   CA.NMCARTAO,
                   BC.CDPICTCONT,
                   CC.CDBANCO ,
                   CC.CDAGENCIA ,
                   CC.CDCONTCORR,
                   TP.NMTPBAIXAREC
                ORDER BY
                   TR.CDTIPORECE
SQL;
        $params = array(
            "P_CDCLIENTE"   => $cdcliente,
            "P_CDFILIAL"    => $cdfilial,
            "P_DTINICIAL"   => $dtinicial,
            "P_DTFINAL"     => $dtfinal,
            "P_TODOS"       => $todos,
            "P_NRLANBOLREC" => $nrlanbolrec,
            "P_VRATUAREC"   => $vratuarec
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getEndereco($cdcliente, $idtpendeclie){
        $sql = <<<SQL
                 SELECT
                     NVL(A.IDTPENDECLIE, 'C') AS IDTPENDECLIE,
                     A.CDCLIENTE,       A.CDPAIS,
                     A.SGESTADO,        A.CDMUNICIPIO,
                     A.DSENDECLIE,      A.NMBAIRCLIE,
                     A.NRCEPCLIE,       A.NRTELECLIE,
                     A.NRRAMALCLIE,     A.NRTELEXCLIE,
                     A.NRFAXCLIE,       A.NMCONTCLIE,
                     A.IDTPENDECLIE,    A.DSEMAILCLIE,
                     A.NRTELECLIE1,     A.NRRAMALCLIE1,
                     A.CDBAIRRO,        A.NRCELULARCLI,
                     A.NRTELRESICLI,    B.NMPAIS,
                     D.NMESTADO,        C.NMMUNICIPIO
                  FROM
                     ENDECLIE A,
                     PAIS B,
                     MUNICIPIO C,
                     ESTADO D
                  WHERE
                     (B.CDPAIS       =  A.CDPAIS       )   AND
                     (C.CDPAIS       =  A.CDPAIS       )   AND
                     (D.CDPAIS       =  A.CDPAIS       )   AND
                     (C.SGESTADO     =  A.SGESTADO     )   AND
                     (D.SGESTADO     =  A.SGESTADO     )   AND
                     (C.CDMUNICIPIO  =  A.CDMUNICIPIO  )   AND
                     (A.CDCLIENTE    =  :P_CDCLIENTE   )   AND
                     (A.IDTPENDECLIE =  NVL(:P_IDTPENDECLIE, 'C'))
SQL;
        $params = array(
            "P_CDCLIENTE"    => $cdcliente,
            "P_IDTPENDECLIE" => $idtpendeclie
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getContReceber($filial, $lancamento, $organizacao){
        $sql = <<<SQL
                 SELECT TR.CDTPCTRECEB, TR.CDCLASSFINA
                    FROM (SELECT MAX(CDARVPROD) AS CDARVPROD
                          FROM ITEMPRODUTO I, PRODUTO P
                          WHERE CDFILIAL    = :P_CDFILIAL AND
                                NRLANCTONF  = :P_NRLANCTONF AND
                                I.NRORG     = :P_NRORG AND
                                I.CDPRODUTO = P.CDPRODUTO AND
                                I.NRORG     = P.NRORG
                          GROUP BY VRTOTAITEM
                          HAVING VRTOTAITEM = (SELECT MAX(VRTOTAITEM)
                                               FROM ITEMPRODUTO
                                               WHERE CDFILIAL    = :P_CDFILIAL AND
                                                     NRLANCTONF  = :P_NRLANCTONF AND
                                                     NRORG       = :P_NRORG)) IT,
                          PRODUTO PR , TIPOCTRECEBER TR
                    WHERE TR.CDTPCTRECEB = PR.CDTPCTRECEB AND
                          PR.CDPRODUTO = (SELECT MAX(CDPRODUTO)
                                          FROM TIPOCTRECEPROD C
                                          WHERE (INSTR(IT.CDARVPROD,RTRIM(C.CDARVPROD)) = 1))
SQL;
        $params = array(
            "P_CDFILIAL"    => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_NRORG" => $organizacao
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getVrDifVencimentoNota($cdfilial, $nrlanctonf, $nrorg, $vrimpostot, $vrsubtraivenc, $vrnotafisc){
        $sql = <<<SQL
        SELECT ABS(NVL(SUM(VRORIGREC),(:P_VRNOTAFISC - :P_VRIMPOSTOT - :P_VRSUBTRAIVENC)) - (:P_VRNOTAFISC - :P_VRIMPOSTOT - :P_VRSUBTRAIVENC)) AS VRDIFVENCIMENTO
           FROM CONTARECEBER
           WHERE CDFILIAL   =  :P_CDFILIAL    AND
                 NRLANCTONF =  :P_NRLANCTONF  AND
                 NRORG      =  :P_NRORG
SQL;

        $params = array(
            'P_CDFILIAL'      => $cdfilial,
            'P_NRLANCTONF'   => $nrlanctonf,
            'P_NRORG'      => $nrorg,
            'P_VRIMPOSTOT'      => $vrimpostot,
            'P_VRSUBTRAIVENC'      => $vrsubtraivenc,
            'P_VRNOTAFISC'      => $vrnotafisc
        );

        return $this->getEntityManager()->getConnection()->fetchColumn($sql, $params);
    }
}